@@ -41,6 +41,9 @@
 
 #ifdef WITH_HILDON
 #include <hildon/hildon.h>
+#if HILDON_CHECK_VERSION(2,2,0)
+#define WITH_HILDON_2_2
+#endif
 #endif
 
 #include "dicto.h"
@@ -56,13 +59,6 @@ G_DEFINE_TYPE(DictoWidget, dicto_widget, GTK_TYPE_HBOX);
 
 #define DICTO_WIDGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), DICTO_WIDGET_TYPE, DictoWidgetPriv))
 
-typedef enum {
-	VIEW_STOPPED,
-	VIEW_PLAYING,
-	VIEW_RECORDING,
-	VIEW_READY
-} DictoWidgetViewMode;
-
 typedef struct _DictoWidgetPriv DictoWidgetPriv; 
 struct _DictoWidgetPriv
 {
@@ -98,8 +94,6 @@ struct _DictoWidgetPriv
 
 	gboolean allow_delete;
 	gboolean seeking;
-
-	DictoWidgetViewMode cmode;
 };
 
 /* Signal IDs */
@@ -123,6 +117,12 @@ enum {
 	NOTE_MAX
 };
 
+enum {
+	DFORMAT_ID=0,
+	DFORMAT_DESC,
+	DFORMAT_S,
+};
+
 static guint signals[LAST_SIGNAL]={ 0 };
 
 static void dicto_widget_set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
@@ -137,7 +137,9 @@ static gboolean dicto_widget_level_cb(GtkWidget *widget, gdouble r, gdouble d, g
 static gboolean dicto_widget_error_cb(GtkWidget *widget, gpointer error, gpointer data);
 
 static void dicto_widget_tree_activate_note_cb(GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, gpointer data);
+#ifdef WITH_HILDON
 static void dicto_widget_tree_tap_note_cb(GtkTreeView *tree_view, GtkTreePath *path, gpointer data);
+#endif
 static void dicto_widget_tree_select_note_cb(GtkTreeSelection *ts, gpointer data);
 
 static gboolean dicto_widget_position_cb(Dicto *d, gdouble left, gpointer data);
@@ -147,7 +149,7 @@ static gboolean dicto_widget_position_press_cb(GtkWidget *widget, GdkEventButton
 static gboolean dicto_widget_position_release_cb(GtkWidget *widget, GdkEventButton *event, gpointer data);
 
 static void dicto_widget_set_length(DictoWidget *dw, gdouble cur_secs, gdouble tot_secs);
-static void dicto_widget_set_view_mode(DictoWidgetPriv *priv, DictoWidgetViewMode mode);
+static void dicto_widget_update_view_mode(DictoWidgetPriv *priv);
 
 /******************************************************************************/
 
@@ -344,13 +346,15 @@ dicto_widget_position_seek_cb(GtkRange *range, gpointer data)
 {
 DictoWidget *dw=(DictoWidget *)data;
 DictoWidgetPriv *priv;
+DictoState s;
 
 g_return_if_fail(IS_DICTO_WIDGET(dw));
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
 g_return_if_fail(priv);
 g_return_if_fail(priv->dicto);
 
-if ((priv->cmode==VIEW_PLAYING || priv->cmode==VIEW_READY) && priv->seeking) {
+s=dicto_get_state(priv->dicto);
+if ((s==DICTO_STATE_PLAYING || s==DICTO_STATE_READY) && priv->seeking) {
 	if (dicto_seek(priv->dicto, gtk_range_get_value(range)))
 		dicto_widget_position_cb(priv->dicto, 0, dw);
 }
@@ -390,24 +394,30 @@ GtkTreeModel *model;
 gchar *file;
 DictoWidget *dw=(DictoWidget *)data;
 DictoWidgetPriv *priv;
+DictoState s;
 
 g_return_if_fail(IS_DICTO_WIDGET(dw));
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
 g_return_if_fail(priv);
 g_return_if_fail(priv->dicto);
 
+s=dicto_get_state(priv->dicto);
+if (s==DICTO_STATE_PLAYING || s==DICTO_STATE_RECORDING)
+	return;
+
 if (!gtk_tree_selection_get_selected(GTK_TREE_SELECTION(ts), &model, &iter))
 	return;
 
 gtk_tree_model_get(model, &iter, NOTE_ID, &file,  -1);
 if (!file)
 	return;
-g_debug("Selected: %s", file);
-if (!dicto_prepare(priv->dicto, file)) {
-	dicto_widget_set_view_mode(priv, VIEW_STOPPED);
-}
-g_free(file);
 
+g_debug("Selected: %s", file);
+
+if (!dicto_prepare(priv->dicto, file))
+	dicto_widget_update_view_mode(priv);
+
+g_free(file);
 }
 
 #ifdef WITH_HILDON
@@ -417,20 +427,29 @@ dicto_widget_tree_tap_note_cb(GtkTreeView *tree_view, GtkTreePath *path, gpointe
 GtkTreeIter iter;
 DictoWidget *dw=(DictoWidget *)data;
 DictoWidgetPriv *priv;
+DictoState s;
 gchar *file;
 
 g_return_if_fail(IS_DICTO_WIDGET(dw));
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
 g_return_if_fail(priv);
+g_return_if_fail(priv->dicto);
 
-g_return_if_fail(gtk_tree_model_get_iter(GTK_TREE_MODEL(priv->file_list), &iter, path));
-gtk_tree_model_get(GTK_TREE_MODEL(priv->file_list), &iter, NOTE_ID, &file, -1);
-if (!file)
+s=dicto_get_state(priv->dicto);
+if (s==DICTO_STATE_PLAYING || s==DICTO_STATE_RECORDING)
 	return;
+
+if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(priv->file_list), &iter, path))
+	return;
+
+gtk_tree_model_get(GTK_TREE_MODEL(priv->file_list), &iter, NOTE_ID, &file, -1);
+g_return_if_fail(!file);
+
 g_debug("File %s tapped", file);
-if (!dicto_prepare(priv->dicto, file)) {
-	dicto_widget_set_view_mode(priv, VIEW_STOPPED);
-}
+
+if (!dicto_prepare(priv->dicto, file))
+	dicto_widget_update_view_mode(priv);
+
 g_free(file);
 }
 #endif
@@ -465,7 +484,7 @@ gtk_tree_model_get(model, &iter, NOTE_ID, &file, -1);
 g_debug("File %s activated", file);
 
 if (!dicto_play(priv->dicto, file))
-	dicto_widget_set_view_mode(priv, VIEW_STOPPED);
+	dicto_widget_update_view_mode(priv);
 
 g_free(file);
 }
@@ -486,7 +505,7 @@ if (dicto_get_position(d, &len, &pos)) {
 	dicto_widget_set_length(dw, pos, len);
 	if (len>0 || pos>0)
 		gtk_range_set_range(GTK_RANGE(priv->position), 0.0, len>=pos ? len : pos);
-	if (priv->cmode==VIEW_PLAYING && !priv->seeking)
+	if (dicto_get_state(priv->dicto)==DICTO_STATE_PLAYING && !priv->seeking)
 		gtk_range_set_value(GTK_RANGE(priv->position), pos);
 }
 
@@ -503,7 +522,7 @@ g_return_val_if_fail(d, TRUE);
 g_return_val_if_fail(data, TRUE);
 
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
-dicto_widget_set_view_mode(priv, VIEW_PLAYING);
+dicto_widget_update_view_mode(priv);
 return TRUE;
 }
 
@@ -517,7 +536,7 @@ g_return_val_if_fail(d, TRUE);
 g_return_val_if_fail(data, TRUE);
 
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
-dicto_widget_set_view_mode(priv, VIEW_RECORDING);
+dicto_widget_update_view_mode(priv);
 return TRUE;
 }
 
@@ -532,7 +551,7 @@ g_return_val_if_fail(data, TRUE);
 
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
 
-dicto_widget_set_view_mode(priv, VIEW_STOPPED);
+dicto_widget_update_view_mode(priv);
 return TRUE;
 }
 
@@ -547,7 +566,7 @@ g_return_val_if_fail(data, TRUE);
 
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
 
-dicto_widget_set_view_mode(priv, VIEW_READY);
+dicto_widget_update_view_mode(priv);
 return TRUE;
 }
 
@@ -561,7 +580,7 @@ GtkTreeIter iter;
 g_debug("%s => %s", (gchar *)key, (gchar *)value);
 
 gtk_list_store_append(store, &iter);
-gtk_list_store_set(store, &iter, NOTE_ID, key, NOTE_NAME, value, -1);
+gtk_list_store_set(store, &iter, NOTE_ID, (gchar *)key, NOTE_NAME, (gchar *)value, -1);
 priv->file_cnt++;
 }
 
@@ -575,9 +594,8 @@ DictoWidgetPriv *priv;
 g_return_val_if_fail(d, TRUE);
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
 
-gtk_list_store_clear(priv->file_list);
 notes=dicto_notes_get_list(d);
-
+gtk_list_store_clear(priv->file_list);
 priv->file_cnt=0;
 g_hash_table_foreach(notes, dicto_widget_append_file, priv);
 
@@ -618,7 +636,7 @@ switch (prop_id) {
 		g_object_ref(d);
 		priv->dicto=d;
 		dicto_widget_updatelist_cb(priv->dicto, dw);
-		dicto_widget_set_view_mode(priv, VIEW_STOPPED);
+		dicto_widget_update_view_mode(priv);
 		g_signal_connect(G_OBJECT(priv->dicto), "position", G_CALLBACK(dicto_widget_position_cb), dw);
 		g_signal_connect(G_OBJECT(priv->dicto), "playing", G_CALLBACK(dicto_widget_playing_cb), dw);
 		g_signal_connect(G_OBJECT(priv->dicto), "recording", G_CALLBACK(dicto_widget_recording_cb), dw);
@@ -692,13 +710,19 @@ gtk_label_set_markup(GTK_LABEL(priv->lbl_len), buffer);
 }
 
 static void
-dicto_widget_set_view_mode(DictoWidgetPriv *priv, DictoWidgetViewMode mode)
+dicto_widget_update_view_mode(DictoWidgetPriv *priv)
 {
-g_return_if_fail(priv);
+DictoState mode;
 
+g_return_if_fail(priv);
+g_return_if_fail(priv->dicto);
+
+mode=dicto_get_state(priv->dicto);
 g_debug("VMode: %d", mode);
 switch (mode) {
-	case VIEW_STOPPED:
+	case DICTO_STATE_ERROR:
+		/* XXX: What should we do ?*/
+	case DICTO_STATE_STOPPED:
 		/* XXX: Add check if we have something to play */
 		dicto_widget_position_clear(priv);
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(priv->level), 0.0);
@@ -714,7 +738,7 @@ switch (mode) {
 		gtk_range_set_update_policy(GTK_RANGE(priv->position), GTK_UPDATE_CONTINUOUS);
 #endif
 	break;
-	case VIEW_READY:
+	case DICTO_STATE_READY:
 		dicto_widget_position_set(priv, 0, 0);
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(priv->level), 0.0);
 		gtk_widget_set_sensitive(priv->btn_record, TRUE);
@@ -727,11 +751,11 @@ switch (mode) {
 		gtk_range_set_value(GTK_RANGE(priv->position), 0.0);
 		gtk_range_set_update_policy(GTK_RANGE(priv->position), GTK_UPDATE_CONTINUOUS);
 	break;
-	case VIEW_RECORDING:
+	case DICTO_STATE_RECORDING:
 		dicto_widget_position_clear(priv);
 		gtk_range_set_range(GTK_RANGE(priv->position), 0.0, 0.1);		
 		dicto_widget_position_set(priv, 0, 0);
-	case VIEW_PLAYING:
+	case DICTO_STATE_PLAYING:
 #ifndef WITH_HILDON_2_2
 		gtk_range_set_update_policy(GTK_RANGE(priv->position), GTK_UPDATE_DELAYED);
 #endif
@@ -741,10 +765,9 @@ switch (mode) {
 		gtk_widget_set_sensitive(priv->btn_delete, FALSE);
 
 		gtk_widget_set_sensitive(priv->file_tree, FALSE);
-		gtk_widget_set_sensitive(priv->position, mode==VIEW_PLAYING);
+		gtk_widget_set_sensitive(priv->position, mode==DICTO_STATE_PLAYING);
 	break;
 }
-priv->cmode=mode;
 }
 
 static gchar *
@@ -811,20 +834,17 @@ return FALSE;
 }
 
 static gboolean
-dicto_widget_level_cb(GtkWidget *widget, gdouble r, gdouble d, gdouble p, gpointer data)
+dicto_widget_level_cb(GtkWidget *widget, gdouble r, gdouble p, gdouble d, gpointer data)
 {
 DictoWidget *dw=(DictoWidget *)data;
 DictoWidgetPriv *priv;
-gfloat tmp;
+gdouble tmp;
 
 priv=DICTO_WIDGET_GET_PRIVATE(dw);
 g_return_val_if_fail(priv, TRUE);
 
-tmp=dicto_normalize_dB(d);
-if (fabsf(priv->ndb-tmp)>0.1 || tmp<0.15) {
-	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(priv->level), dicto_normalize_dB(d));
-	priv->ndb=tmp;
-}
+tmp=(r+p+p+d)/4;
+gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(priv->level), dicto_normalize_dB(tmp));
 
 return FALSE;
 }
@@ -856,7 +876,7 @@ priv=DICTO_WIDGET_GET_PRIVATE(dw);
 g_return_val_if_fail(priv, TRUE);
 
 g_debug("Ready to play");
-dicto_widget_set_view_mode(priv, VIEW_READY);
+dicto_widget_update_view_mode(priv);
 
 return TRUE;
 }
@@ -876,7 +896,7 @@ if (!f)
 	return TRUE;
 
 if (!dicto_play(priv->dicto, f))
-	dicto_widget_set_view_mode(priv, VIEW_STOPPED);
+	dicto_widget_update_view_mode(priv);
 
 g_free(f);
 return TRUE;
@@ -914,7 +934,7 @@ if (!file) {
 g_signal_emit(dw, signals[SIGNAL_DELETE_QUERY], 0, file, &ret);
 if (ret) {
 	dicto_delete(priv->dicto, file);
-	dicto_widget_set_view_mode(priv, VIEW_STOPPED);
+	dicto_widget_update_view_mode(priv);
 }
 
 return TRUE;
@@ -982,9 +1002,8 @@ while (i<FORMAT_LAST) {
 		i++;
 		continue;
 	}
-		
 	gtk_list_store_append(ls, &iter);
-	gtk_list_store_set(ls, &iter, 0, f->id, 1, f->desc, 2, f, -1);
+	gtk_list_store_set(ls, &iter, DFORMAT_ID, f->id, DFORMAT_DESC, f->desc, DFORMAT_S, f, -1);
 	i++;
 }
 
