@@ -73,7 +73,7 @@ enum {
 	SIGNAL_SEEK,
 	SIGNAL_LEVEL,
 	LAST_SIGNAL
-};
+} DictoSignal;
 
 /* Property IDs */
 enum {
@@ -87,7 +87,8 @@ enum {
 	PROP_FORMAT,
 	PROP_POS_INTERVAL,
 	PROP_TIMEFMT,
-};
+	PROP_STATE,
+} DictoProp;
 
 typedef struct _dicto_pipeline dicto_pipeline;
 struct _dicto_pipeline {
@@ -115,6 +116,8 @@ struct _DictoPrivate {
 	gchar *gst_source;
 	gchar *timefmt;
 	DictoFormats format;
+	GHashTable *formats;
+	DictoState state;
 	gint pos_sid;
 	guint pos_int;
 	gdouble pos;
@@ -125,6 +128,7 @@ struct _DictoPrivate {
 	gdouble rforsecs;
 	gboolean has_decodebin;
 	GHashTable *notes;
+	GHashTable *tags;
 	dicto_pipeline play;
 	dicto_pipeline record;
 };
@@ -135,14 +139,13 @@ static guint signals[LAST_SIGNAL]={ 0 };
 
 #define UNREF_ELEMENT(e) { if (np->e) { gst_object_unref(np->e); np->e=NULL; } }
 
-static gboolean dicto_create_play_pipeline(Dicto *d);
-static gboolean dicto_create_record_pipeline(Dicto *d, DictoFormats nf);
+static gboolean dicto_create_play_pipeline(Dicto *d, GError **error);
+static gboolean dicto_create_record_pipeline(Dicto *d, DictoFormats nf, GError **error);
 static gboolean dicto_notes_load_list(Dicto *d, gboolean s);
 
 static void
 dicto_dispose(GObject *object)
 {
-g_debug("Dispose");
 G_OBJECT_CLASS(dicto_parent_class)->dispose(object);
 }
 
@@ -166,9 +169,10 @@ DictoPrivate *p=GET_PRIVATE(d);
 
 g_return_if_fail(d);
 
-g_debug("Finalize");
 if (p->pos_sid!=0)
 	g_source_remove(p->pos_sid);
+if (p->tags)
+	g_hash_table_destroy(p->tags);
 
 gst_element_set_state(p->play.pipeline, GST_STATE_NULL);
 gst_element_set_state(p->record.pipeline, GST_STATE_NULL);
@@ -260,6 +264,9 @@ switch (prop_id) {
 	case PROP_TIMEFMT:
 		g_value_set_string(value, p->timefmt);
 	break;
+	case PROP_STATE:
+		g_value_set_uint(value, p->state);
+	break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	break;
@@ -269,22 +276,32 @@ switch (prop_id) {
 static void
 dicto_constructed(GObject *object)
 {
+guint i=0;
 Dicto *d=DICTO(object);
 DictoPrivate *p=GET_PRIVATE(d);
+DictoFormat *f;
+
+p->formats=g_hash_table_new(g_direct_hash, g_direct_equal);
+
+while (dicto_formats[i].channels>0) {
+	f=&dicto_formats[i];
+	dicto_recording_format_add(d, f->id, f);
+	i++;
+}
 
-g_debug("Constructed");
 p->rms=0;
 p->timefmt=g_strdup(DEF_TIME_FORMAT);
 p->pos_int=POS_INT_DEFAULT;
-dicto_create_play_pipeline(d);
-dicto_create_record_pipeline(d, p->format);
+p->tags=NULL;
+ 
+dicto_create_play_pipeline(d, NULL);
+dicto_create_record_pipeline(d, p->format, NULL);
 dicto_notes_load_list(d, FALSE);
 }
 
 static void 
 dicto_init(Dicto *d)
 {
-g_debug("INIT");
 }
 
 static void
@@ -359,6 +376,14 @@ g_object_class_install_property(object_class, PROP_CURFILE, pspec);
 pspec=g_param_spec_string("time-format", "File format", "File timestamp format", DEF_TIME_FORMAT, G_PARAM_READWRITE);
 g_object_class_install_property(object_class, PROP_TIMEFMT, pspec);
 
+/**
+ * Dicto:state
+ *
+ * The current DictoState state of #Dicto,
+ */
+pspec=g_param_spec_uint("state", "State", "Current state", 0, 8, 0, G_PARAM_READABLE);
+g_object_class_install_property(object_class, PROP_STATE, pspec);
+
 signals[SIGNAL_POSITION]=g_signal_new("position", G_OBJECT_CLASS_TYPE(object_class),
 	G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET(DictoClass, position_update),
 	NULL, NULL, g_cclosure_marshal_VOID__DOUBLE, G_TYPE_NONE, 1, G_TYPE_DOUBLE);
@@ -405,6 +430,17 @@ dicto_normalize_dB(gdouble dB)
 return pow(10, dB/20);
 }
 
+DictoState
+dicto_get_state(Dicto *d)
+{
+DictoPrivate *p;
+
+g_return_val_if_fail(IS_DICTO(d), FALSE);
+p=GET_PRIVATE(d);
+
+return p->state;
+}
+
 static gboolean
 dicto_position_cb(gpointer data)
 {
@@ -447,6 +483,35 @@ if (ena && p->pos_sid==0)
 	p->pos_sid=g_timeout_add(p->pos_int, (GSourceFunc)dicto_position_cb, d);
 }
 
+/**
+ * dicto_recording_format_add:
+ * 
+ * Add a recording format.
+ *
+ */
+gboolean
+dicto_recording_format_add(Dicto *d, guint f_id, DictoFormat *f)
+{
+DictoPrivate *p=GET_PRIVATE(d);
+
+g_hash_table_insert(p->formats, GINT_TO_POINTER(f_id), f);
+return TRUE;
+}
+
+/**
+ * dicto_recording_format_remove:
+ *
+ * Remove a recording format.
+ *
+ */
+static gboolean
+dicto_recording_format_remove(Dicto *d, guint f_id)
+{
+DictoPrivate *p=GET_PRIVATE(d);
+
+return g_hash_table_remove(p->formats, GINT_TO_POINTER(f_id));
+}
+
 /**
  * dicto_set_basedir:
  * 
@@ -509,13 +574,19 @@ dicto_load_tag(const GstTagList *list, const gchar *tag, gpointer data)
 {
 gchar *v;
 GType t;
+Dicto *d=(Dicto *)data;
+DictoPrivate *p=GET_PRIVATE(d);
+
+g_return_if_fail(d);
+g_return_if_fail(p);
+g_return_if_fail(p->tags);
 
 t=gst_tag_get_type(tag);
 switch (t) {
 	case G_TYPE_STRING:
 	if (gst_tag_list_get_string(list, tag, &v)) {
 		g_debug("Tag: %s = %s", tag, v);
-		g_free(v);
+		g_hash_table_insert(p->tags, g_strdup(tag), v);
 	}
 	break;
 	default:;
@@ -546,7 +617,6 @@ for (i=0;i<channels;++i) {
 	*rms_dB+=g_value_get_double(gst_value_list_get_value(gst_structure_get_value(s, "rms"), i));
 	*peak_dB+=g_value_get_double(gst_value_list_get_value (gst_structure_get_value (s, "peak"), i));
 	*decay_dB+=g_value_get_double(gst_value_list_get_value (gst_structure_get_value (s, "decay"), i));
-	g_debug("RMS-%d: %f dB, peak: %f dB, decay: %f dB", i, *rms_dB, *peak_dB, *decay_dB);
 }
 
 p->decay=*decay_dB/channels;
@@ -564,22 +634,24 @@ GError *err;
 GstTagList *tags;
 const GstStructure *s;
 const gchar *name;
-
 dicto_pipeline *np=(dicto_pipeline *)data;
+Dicto *d=(Dicto *)np->owner;
+DictoPrivate *p=GET_PRIVATE(d);
 
 switch (GST_MESSAGE_TYPE(msg)) {
 	case GST_MESSAGE_EOS:
 		np->active=FALSE;
-   		g_debug("EOS");
 		if (np->rec) {
 			gst_element_set_state(np->pipeline, GST_STATE_NULL);
 			dicto_position_update(DICTO(np->owner), FALSE);
-			g_signal_emit(np->owner, signals[SIGNAL_STOPPED_RECORD], 0, NULL);
+			p->state=DICTO_STATE_STOPPED;
+			g_signal_emit(G_OBJECT(d), signals[SIGNAL_STOPPED_RECORD], 0, NULL);
 		} else {
 			gst_element_set_state(np->pipeline, GST_STATE_PAUSED);
-			dicto_position_update(DICTO(np->owner), FALSE);
+			dicto_position_update(d, FALSE);
+			p->state=DICTO_STATE_STOPPED;
 			g_signal_emit(np->owner, signals[SIGNAL_STOPPED_PLAY], 0, NULL);
-			dicto_seek(DICTO(np->owner), 0.0);
+			dicto_seek(d, 0.0);
 		}
 	break;
 	case GST_MESSAGE_ERROR:
@@ -588,11 +660,10 @@ switch (GST_MESSAGE_TYPE(msg)) {
 		g_free(debug);
 		np->active=FALSE;
 		dicto_position_update(DICTO(np->owner), FALSE);
+		gst_element_set_state(np->pipeline, GST_STATE_NULL);
+		p->state=DICTO_STATE_ERROR;
 		g_signal_emit(np->owner, signals[SIGNAL_ERROR], 0, err, NULL);
 		g_error_free(err);
-		gst_element_set_state(np->pipeline, GST_STATE_NULL);
-		/* send stop signal as we go to null state on errors */
-		g_signal_emit(np->owner, np->rec ? signals[SIGNAL_STOPPED_RECORD] : signals[SIGNAL_STOPPED_PLAY], 0, NULL);
 	break;
 	case GST_MESSAGE_STATE_CHANGED:
 		if (GST_MESSAGE_SRC(msg)!=GST_OBJECT(np->pipeline))
@@ -603,6 +674,7 @@ switch (GST_MESSAGE_TYPE(msg)) {
 		if (np->rec && newstate==GST_STATE_READY && oldstate==GST_STATE_PLAYING) {
 			gst_element_set_state(np->pipeline, GST_STATE_NULL);
 			g_object_set(G_OBJECT(np->sink), "location", "", NULL);
+			p->state=DICTO_STATE_STOPPED;
 			g_signal_emit(np->owner, signals[SIGNAL_STOPPED_RECORD], 0, NULL);
 		}
 		g_debug("GST: [%d] state: O:%d N:%d P:%d", np->rec, oldstate, newstate, pending);
@@ -680,8 +752,9 @@ return c;
  * It creates a temporary pipeline first, and if everything is ok then the current one will be replaced.
  */
 static gboolean
-dicto_create_record_pipeline(Dicto *d, DictoFormats nf)
+dicto_create_record_pipeline(Dicto *d, DictoFormats nf, GError **error)
 {
+GError *err=NULL;
 GstBus *bus;
 GstCaps *caps;
 GstElement *enext, *ac;
@@ -690,7 +763,6 @@ DictoFormat *f;
 DictoPrivate *p=GET_PRIVATE(d);
 dicto_pipeline *np=&p->record;
 
-g_debug("GST: Creating record pipeline, format: %d", nf);
 g_return_val_if_fail(p->gst_source, FALSE);
 
 if (p->record.active==TRUE)
@@ -706,21 +778,17 @@ filter=NULL;
 f=&dicto_formats[nf];
 g_return_val_if_fail(f->id==nf, FALSE);
 
-g_debug("* %d-bits, %d-channels, rate: %d", f->bits, f->channels, f->rate);
-
 if (f->muxer) {
 	muxer=gst_element_factory_make(f->muxer, "muxer");
 	g_return_val_if_fail(muxer, FALSE);
 }
 
 if (f->encoder) {
-	g_debug("Encoder: %s", f->encoder);
 	encoder=gst_element_factory_make(f->encoder, "encoder");
 	g_return_val_if_fail(encoder, FALSE);
 }
 
 if (f->filter) {
-	g_debug("Filter: %s", f->filter);
 	filter=gst_element_factory_make(f->filter, "filter");
 	g_return_val_if_fail(filter, FALSE);
 }
@@ -734,9 +802,15 @@ if (!queue)
 	goto error_rec;
 gst_bin_add(GST_BIN(pipeline), queue);
 
-src=gst_element_factory_make(p->gst_source, "audiosrc");
-if (!src)
-	goto error_rec;
+if (strchr(p->gst_source, '!')==NULL && (strchr(p->gst_source, ' ')==NULL)) {
+	src=gst_element_factory_make(p->gst_source, "audiosrc");
+	if (!src)
+		goto error_rec;
+} else {
+	src=gst_parse_bin_from_description(p->gst_source, TRUE, &err);
+	if (!src || err!=NULL)
+		goto error_rec;
+}
 gst_bin_add(GST_BIN(pipeline), src);
 
 sink=gst_element_factory_make("filesink", "fsink");
@@ -847,6 +921,11 @@ return TRUE;
 
 error_rec: ;
 
+if (err!=NULL) {
+	g_debug("Error message: %s", err->message);
+	g_propagate_error(error, err);
+}
+
 dicto_pipeline_free(np);
 return FALSE;
 }
@@ -883,14 +962,13 @@ return FALSE;
 #endif
 
 static gboolean
-dicto_create_play_pipeline(Dicto *d)
+dicto_create_play_pipeline(Dicto *d, GError **error)
 {
 GstBus *bus;
 DictoPrivate *p=GET_PRIVATE(d);
 GstElement *bsink, *filter, *queue;
 dicto_pipeline *np=&p->play;
 
-g_debug("GST: Creating playback pipeline");
 g_return_val_if_fail(p->gst_sink, FALSE);
 
 np->pipeline=gst_pipeline_new("pipeline");
@@ -981,7 +1059,6 @@ g_return_val_if_fail(e, FALSE);
 g_return_val_if_fail(p->basedir, FALSE);
 
 if (!file) {
-	g_debug("Clearing file");
 	g_object_set(G_OBJECT(e), "location", "", NULL);
 	if (p->cfile)
 		g_free(p->cfile);
@@ -1043,7 +1120,6 @@ while ((f=g_dir_read_name(ddir))!=NULL) {
 		g_free(tmp);
 		continue;
 	}
-	g_debug("File: %s", f);
 	g_hash_table_insert(p->notes, g_strdup(f), g_strdup(f));
 	g_free(tmp);
 }
@@ -1063,6 +1139,23 @@ g_return_val_if_fail(d, FALSE);
 return p->notes;
 }
 
+/**
+ * dicto_get_tag:
+ * @d: A #Dicto object
+ * @tag: tag to return value for
+ *
+ * Returns: A tag value if set, NULL otherwise.
+ *
+ * Since: 1.0.9
+ */
+gchar *
+dicto_get_tag(Dicto *d, const gchar *tag)
+{
+DictoPrivate *p=GET_PRIVATE(d);
+
+return g_hash_table_lookup(p->tags, tag);
+}
+
 /**
  * dicto_refresh:
  * @d: A #Dicto object
@@ -1123,9 +1216,10 @@ if (gst_element_set_state(p->play.pipeline, GST_STATE_PLAYING)==GST_STATE_CHANGE
 	return FALSE;
 }
 
-g_debug("Playing: %s", p->cfile);
 p->play.active=TRUE;
 dicto_position_update(d, TRUE);
+
+p->state=DICTO_STATE_PLAYING;
 g_signal_emit(d, signals[SIGNAL_PLAYING], 0, NULL);
 return TRUE;
 }
@@ -1159,6 +1253,10 @@ if (p->record.active==TRUE)
 
 g_return_val_if_fail(file, FALSE);
 
+if (p->tags)
+	g_hash_table_destroy(p->tags);
+p->tags=g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+
 gst_element_set_state(p->play.pipeline, GST_STATE_NULL);
 dicto_set_filename(d, p->play.src, file);
 if (gst_element_set_state(p->play.pipeline, GST_STATE_PAUSED)==GST_STATE_CHANGE_FAILURE) {
@@ -1173,7 +1271,7 @@ if (gst_element_get_state(p->play.pipeline, &current, NULL, GST_SECOND/2)==GST_S
 	return FALSE;
 }
 
-g_debug("Prepared: %s", p->cfile);
+p->state=DICTO_STATE_READY;
 g_signal_emit(d, signals[SIGNAL_READY], 0, NULL);
 return TRUE;
 }
@@ -1234,8 +1332,7 @@ DictoPrivate *p=GET_PRIVATE(d);
 g_return_val_if_fail(d, FALSE);
 g_return_val_if_fail(file, FALSE);
 
-if (!p->record.pipeline)
-	return FALSE;
+g_return_val_if_fail(p->record.pipeline, FALSE);
 
 if (p->record.active==TRUE)
 	return FALSE;
@@ -1248,6 +1345,11 @@ if (p->cfile) {
 	p->cfile=NULL;
 }
 
+if (p->tags) {
+	g_hash_table_destroy(p->tags);
+	p->tags=NULL;
+}
+
 p->rforsecs=secs;
 gst_element_set_state(p->record.pipeline, GST_STATE_NULL);
 dicto_set_filename(d, p->record.sink, file);
@@ -1259,10 +1361,10 @@ if (gst_element_set_state(p->record.pipeline, GST_STATE_PLAYING)==GST_STATE_CHAN
 p->record.active=TRUE;
 
 dicto_position_update(d, TRUE);
+p->state=DICTO_STATE_RECORDING;
 g_signal_emit(d, signals[SIGNAL_RECORDING], 0, NULL);
 g_idle_add(dicto_refresh_idle_cb, d);
 
-g_debug("Recording");
 return TRUE;
 }
 
@@ -1282,19 +1384,22 @@ return dicto_record_timeout(d, file, -1);
 }
 
 /**
- * dicto_delete:
+ * dicto_delete_full:
  * @d: A #Dicto object
  * @file: A note to delete, must be an existing note file name
+ * @err: Pointer to #GError or NULL
  *
  * Returns: %TRUE if note was deleted. %FALSE on error.
+ *
+ * Since: 1.0.8
  */
 gboolean
-dicto_delete(Dicto *d, const gchar *file)
+dicto_delete_full(Dicto *d, const gchar *file, GError **err)
 {
 DictoPrivate *p=GET_PRIVATE(d);
 gchar *tmp;
-gint r;
 
+g_return_val_if_fail(err==NULL || *err==NULL, FALSE);
 g_return_val_if_fail(d, FALSE);
 g_return_val_if_fail(p->notes, FALSE);
 g_return_val_if_fail(p->basedir, FALSE);
@@ -1306,12 +1411,21 @@ if (g_path_is_absolute(file))
 if (g_hash_table_lookup(p->notes, file)==NULL)
 	return FALSE;
 
+if (p->state==DICTO_STATE_PLAYING || p->state==DICTO_STATE_RECORDING)
+	return FALSE;
+
 tmp=g_build_filename(p->basedir, file, NULL);
 g_return_val_if_fail(tmp, FALSE);
-r=g_unlink(tmp);
-if (r!=0)
-	g_warning("Unlink of note %s (%s) failed", file, tmp);
-g_debug("Removed: %s", tmp);
+
+if (g_file_test(tmp, G_FILE_TEST_EXISTS) && g_unlink(tmp)==-1) {
+	g_set_error (err,	G_FILE_ERROR,
+		g_file_error_from_errno(errno),
+		"Existing note '%s' could not be removed: g_unlink() failed: %s",
+		file, g_strerror(errno));
+	g_free(tmp);
+	return FALSE;
+}
+
 g_free(tmp);
 
 if (g_hash_table_remove(p->notes, file)==FALSE) {
@@ -1326,10 +1440,26 @@ if (p->play.pipeline) {
 	dicto_set_filename(d, p->play.src, NULL);
 }
 
+p->state=DICTO_STATE_STOPPED;
 g_signal_emit(d, signals[SIGNAL_NOTES_REFRESH], 0, NULL);
 return TRUE;
 }
 
+/**
+ * dicto_delete:
+ * @d: A #Dicto object
+ * @file: A note to delete, must be an existing note file name
+ *
+ * Simple version of dicto_delete_full
+ *
+ * Returns: FALSE on error, TRUE otherwise.
+ */
+gboolean
+dicto_delete(Dicto *d, const gchar *file)
+{
+return dicto_delete_full(d, file, NULL);
+}
+
 /**
  * dicto_stop:
  * @d: A #Dicto object
@@ -1348,16 +1478,8 @@ GstState pending;
 if (p->record.pipeline) {
 	gst_element_get_state(p->record.pipeline, &current, &pending, GST_CLOCK_TIME_NONE);
 	if (current==GST_STATE_PLAYING) {
-		g_debug("Stop: recording");
 		dicto_position_update(d, FALSE);
-#if 0
-		gst_element_set_state(p->record.pipeline, GST_STATE_READY);
-		p->record.active=FALSE;
-		p->rms=0;
-		g_signal_emit(d, signals[SIGNAL_STOPPED_RECORD], 0, NULL);
-#else
 		gst_element_send_event(p->record.pipeline, gst_event_new_eos());
-#endif
 		return TRUE;
 	}
 }
@@ -1365,11 +1487,11 @@ if (p->record.pipeline) {
 if (p->play.pipeline) {
 	gst_element_get_state(p->play.pipeline, &current, &pending, GST_CLOCK_TIME_NONE);
 	if (current==GST_STATE_PLAYING || current==GST_STATE_PAUSED) {
-		g_debug("Stop: playing");
 		dicto_position_update(d, FALSE);
 		gst_element_set_state(p->play.pipeline, GST_STATE_READY);
 		p->play.active=FALSE;
 		p->rms=0;
+		p->state=DICTO_STATE_STOPPED;
 		g_signal_emit(d, signals[SIGNAL_STOPPED_PLAY], 0, NULL);
 		return TRUE;
 	}
@@ -1405,10 +1527,12 @@ return (p->record.active) ? dicto_stop(d) : dicto_record(d, file);
 
 /**
  * dicto_get_format:
+ * @d: A #Dicto object.
+ * @fs: Format id to query for.
  *
  * Request information about format. 
  *
- * Returns: A pointer to a #DictoFormat. NULL if requested format can't be used.
+ * Returns: A pointer to a #DictoFormat. NULL if requested format can't be used or not found.
  */
 const DictoFormat *
 dicto_get_format(Dicto *d, DictoFormats fs)
@@ -1417,9 +1541,13 @@ DictoFormat *f;
 DictoPrivate *p=GET_PRIVATE(d);
 
 g_return_val_if_fail(d, NULL);
-g_return_val_if_fail(f>=FORMAT_LAST, NULL);
 
-f=&dicto_formats[fs];
+f=g_hash_table_lookup(p->formats, GINT_TO_POINTER(fs));
+if (!f)
+	return NULL;
+
+g_return_val_if_fail(f->id==fs, NULL);
+
 if (f->encoder && !p->has_decodebin) {
 	return NULL;
 } else if (f->encoder && p->has_decodebin) {
@@ -1487,7 +1615,7 @@ dicto_set_format(Dicto *d, DictoFormats nf)
 DictoPrivate *p=GET_PRIVATE(d);
 if (p->format==nf)
 	return TRUE;
-return dicto_create_record_pipeline(d, nf);
+return dicto_create_record_pipeline(d, nf, NULL);
 }
 
 /**
